/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: selfish_detour.c,v 1.34 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
/* ===================================
   SELFISH DETOUR NOISE INJECTION CODE

   add same duration noise constanly with or without randomized start
   =================================== */

#include <stdlib.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>

#include "selfish_utils.h"

static long detour_duration;
static int _detour_signal_handler_cnt = 0;


static const long detour_types_constant[][2] = {
  /* interval, duration */
  { 100*1000,  16 },  
  { 100*1000,  20 },  // every 10 msec , 20 usec delay
  { 100*1000,  30 },
  { 100*1000,  50 },
  {  10*1000,  16 },  
  {  10*1000,  20 },  // every 10 msec , 20 usec delay
  {  10*1000,  30 },
  {  10*1000,  50 },
  {   1*1000,  16 },
  {   1*1000,  20 },
  {   1*1000,  30 },
  {   1*1000,  50 },

  /* 
     Here is the final modification during BGW testing.
     do we use this table as default?
  { 100*1000,  50 },  
  { 100*1000,  100 },
  { 100*1000,  200 },
  {  10*1000,  50 },  
  {  10*1000,  100 },
  {  10*1000,  200 },
  {   1*1000,  50 },
  {   1*1000,  100 },
  {   1*1000,  200 },
   */
};


#define SIGNAL_OVERHEAD_US  (15)

static void detour_signal_handler(int sig)
{
  unsigned long long s,b,duration_tick;
  double x = 0.0;


_detour_signal_handler_cnt ++;

  duration_tick = (detour_duration - SIGNAL_OVERHEAD_US)*TICKS_PER_US;
  if( duration_tick < 0 ) return ;

  s = rdtsc();
  b = s;
  /* 524 is an experimental weight to adjust for signal overhead */
  while ( (b-s) < duration_tick - 524 ) {
    x = x + 1.00000001;
    b = rdtsc();
  }
}


//********************************************************************
// Simple timer/alarm setup
// to make it to be able to call from fortran code
// this function uses reference
// *mode 
//    0 : does nothing
//    1 : causes timer update by setting very long interval
//    2 : fixed interval 
void start_detour_injection(int *rank,
			    int *mode,
			    int *detour_type,
			    int *randomized_start )
{
  struct itimerval itv;
  int myrank;



  if( *mode == 0 ) {
    return;
  }

  myrank = *rank;

  // We use the ``Mersenne Twister'' random number generator MT19937
  // You can seed with any uint32, but the best are odds in 0..(2^32 - 1)
  // To make sure that each node starts their random sequence at a 
  // different place, we use the rank in the seed

  seedMT(4357 + (myrank + myrank));  // this should be an odd number

  /* 
     random delay(up to 1sec) before benchmark
   */
  if( *randomized_start ) {
    unsigned long long delay_ticks = randomMT() % (700*1000*1000);
    unsigned long long s,b;
    double x = 0.0;

    s = rdtsc();
    b = s;
    while ( (b-s) < delay_ticks) {
      x = x + 1.00000001;
      b = rdtsc();
    }
  }
  

  if(*mode == 1 ) {
    itv.it_value.tv_sec   = 60*60*24; /* 24 hours */
    itv.it_value.tv_usec  = 0;
    itv.it_interval       = itv.it_value;
  } else if(*mode == 2 ) {
    itv.it_value.tv_sec   = 0;
    itv.it_value.tv_usec  = detour_types_constant[*detour_type][0];
    itv.it_interval       = itv.it_value;
    detour_duration = detour_types_constant[*detour_type][1];
  }

  _detour_signal_handler_cnt=0;

  signal( SIGALRM, detour_signal_handler);
  setitimer(ITIMER_REAL, &itv, 0);
}



/*
  some helper functions
*/
const int get_detour_count()
{
  return _detour_signal_handler_cnt;
}
const int get_n_detour_types(int *mode)
{
  if( *mode == 0 || *mode == 1 ) return 1;
  else if( *mode == 2 ) {
    return sizeof( detour_types_constant) / ( 2 * sizeof(long));
  }
  return 0;
}

const long get_detour_interval_usec(int *mode, int *detour_type)
{
  if(*mode == 0 ) {
    return 0;
  } else if(*mode == 1 ) {
    return  6135667;  /* = 2^32/700 */
  } else if(*mode == 2 ) {
    return detour_types_constant[*detour_type][0];
  }
  return 0;
}

const long get_detour_duration_usec(int *mode, int *detour_type)
{
  if(*mode == 0 ) {
    return 0;
  } else if( *mode == 1 ) {
    return 2;
  } else if( *mode == 2 ) {
    return detour_types_constant[*detour_type][1];
  }
  return 0;
}

