/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: selfish_detour_playback.c,v 1.3 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
/* ===================================
   SELFISH DETOUR NOISE INJECTION CODE

   add same duration noise constanly with or without randomized start
   =================================== */

#include <stdlib.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>

#include "selfish_utils.h"

/**
 *
 * for detour playback mode  (still experiment code)
 *
 * extern int  selfish_detour_playback_finished;
 *
 * void read_selfish_detour_data(const char* filename);
 * 
 * void selfish_detour_playback_init(void);
 * void selfish_detour_playback_finalize(void);
 *
 * current status: implemented ... testing
 *
 * limitation and assumption (for BG/L CNK)
 *
 * - the system interval timer never expire before the requested time.
 * - duration  > ~20us
 * - interval >> duration
 * - interval < 1 second
 *
 */

#ifdef __blrts__
/* these value are experimental value */
#define  SIGNAL_OVERHEAD_TICKS      (10325) /* approx. 14.8 usec */
#define  SETITIMER_OVERHEAD_TICKS   (2535)  /* approx.  3.4 usec */
#else
/* dummy */
#define  SIGNAL_OVERHEAD_TICKS      (10325)
#define  SETITIMER_OVERHEAD_TICKS   (2535)
static int       ticks_per_usec;

#endif


/**
 * detour_playback_array holds detour time series data, loaded from file
 * which is recorded by record_selft_detour program.
 *
 * detour_playback_array[n*2+0]    n'th detour's start time in tick
 * detour_playback_array[n*2+1]    n'th detour's duration in tick
 */

static int       n_detour_playback_array;
static int       detour_playback_array_pos=0;
static uint64_t* detour_playback_array=(uint64_t*)0;
static uint64_t  detour_playback_start_tick=0;
				       
volatile int selfish_detour_playback_finished;
volatile int selfish_detour_playback_cnt;
volatile uint64_t selfish_detour_playback_elapsed;


/**
 * simple timer stop function 
 */
static __inline__ void detour_playback_stop_timer(void)
{
  struct itimerval itv;
  itv.it_value.tv_sec   = 0;
  itv.it_value.tv_usec  = 0;
  itv.it_interval       = itv.it_value;
  setitimer(ITIMER_REAL, &itv, 0);
}



/**
 * read_detour_data()
 *
 * @param filename   playback data filename
 */
void read_selfish_detour_data(const char* filename)
{
  FILE *fp;
  char buf[80];
  uint64_t t, d;
  int n;

  fp = fopen( filename, "r" );
  if( !fp ) {
    printf("Unable to open %s\n", filename);
    exit(1);
  }

  if(detour_playback_array) {
    /* stop timer */
    detour_playback_stop_timer();
    
    free( detour_playback_array );
  }

  n_detour_playback_array = 0;
  detour_playback_array_pos=0;
  detour_playback_array=(uint64_t*)0;
  n = 0;

  while( fgets(buf, sizeof(buf), fp) != (char*)0 ) {
    if( isdigit( buf[0] ) ) {
      if( n_detour_playback_array == 0 ) {
	printf("wrong data\n");
	exit(1);
      }

      sscanf( buf, "%llu %llu", &t, &d);

      detour_playback_array[n*2+0] = t;
      detour_playback_array[n*2+1] = d;
      n++;

    } else if( buf[0] == '#' ) {
      char* n_segs_str = "# N_SEGS=";
      if( strncmp( buf, n_segs_str, strlen(n_segs_str) )==0 ) {
	n_detour_playback_array = atoi( buf+strlen(n_segs_str) );
	detour_playback_array = (uint64_t*)malloc( sizeof(uint64_t)*n_detour_playback_array * 2 );
	if( ! detour_playback_array  ) {
	  printf("Unable to allocate memory\n");
	  exit(1);
	}
      }
    }
  }

  if( n!=n_detour_playback_array ) {
    printf("seems wrong : n=%d   n_detour_playback_array=%d\n",
	   n, n_detour_playback_array);
  }
  
  fclose(fp);
  return;
}




/**
 * detour_playback_handler() adjusts interval time everytime because
 * setitimer is not accurate.
 *
 */ 

static void detour_playback_handler(int sig)
{
  struct   itimerval itv;
  int      rc, i;
  double   x;
  uint64_t current, elapsed;
  uint64_t next_interval, detour_tick, start;

  selfish_detour_playback_cnt ++;

  current = rdtsc();

  detour_playback_stop_timer(); /* stop timer until adjustment has completed */

  if( detour_playback_array_pos >=
      n_detour_playback_array - 1 ) {
    selfish_detour_playback_finished=1;
    return;
  }

  elapsed = current - detour_playback_start_tick;
  // next_interval = detour_playback_array[(detour_playback_array_pos+1)*2+0] - elapsed;

  next_interval = 
    detour_playback_array[(detour_playback_array_pos+1)*2+0] -
    detour_playback_array[(detour_playback_array_pos+0)*2+0];

  next_interval -= 
    ( (current-detour_playback_start_tick) -
      detour_playback_array[(detour_playback_array_pos+0)*2+0] ); 

  next_interval -= 8000; // weight. adjusted based on experimental. dirty
  
  if( next_interval <= 0 ) {
    printf("something wrong. stop timer\n");
    printf("detour_playback_array[(%d+1)*2+0]=%llu\n",
	   detour_playback_array_pos,
	   detour_playback_array[ (detour_playback_array_pos+1)*2+0] );
    printf("elapsed=%llu\n", elapsed);
    selfish_detour_playback_finished=1;
    return;
  }

  detour_tick = 
    detour_playback_array[(detour_playback_array_pos)*2+1] 
    - SIGNAL_OVERHEAD_TICKS - SETITIMER_OVERHEAD_TICKS 
    - 2000; // weight. adjusted based on experimental.  dirty

  if( detour_tick > 0 ) {
    /**
     * fake detour
     */
    selfish_detour_playback_elapsed += detour_tick;

    i=0;  x=0.0;
    start = rdtsc();
    while( (rdtsc() - start) < detour_tick ) {
      // do a floating point calculation here
      if( (i%2)==0 ) 
	x += 1.0/(1.0+2.0*(double)i);
      else  
	x -= 1.0/(1.0+2.0*(double)i);
      i++;
    }
    next_interval -= detour_tick; 
  }



  detour_playback_array_pos++;

  itv.it_value.tv_sec   = 0;



#ifdef __blrts__
  itv.it_value.tv_usec  = next_interval/699;
#else
  itv.it_value.tv_usec  = next_interval/ticks_per_usec;
#endif
  itv.it_interval       = itv.it_value;

  rc = setitimer(ITIMER_REAL, &itv, 0);
  assert( rc == 0 );


  return;
}


void selfish_detour_playback_finalize(void)
{
  selfish_detour_playback_finished=1;
  detour_playback_stop_timer();
}


/**
 */

void selfish_detour_playback_init(void)
{
  struct   itimerval itv;
  int rc;

  detour_playback_array_pos=0;

#ifndef __blrts__
  ticks_per_usec = (cpufreq()/1000)/1000;
#endif

  itv.it_value.tv_sec   = 0;

#ifdef __blrts__
  itv.it_value.tv_usec  = detour_playback_array[detour_playback_array_pos*2+0]/699;
#else
  itv.it_value.tv_usec  = detour_playback_array[detour_playback_array_pos*2+0]/ticks_per_usec;
#endif

  itv.it_interval       = itv.it_value;

  signal( SIGALRM, detour_playback_handler );

  selfish_detour_playback_finished=0;
  selfish_detour_playback_cnt=0;
  selfish_detour_playback_elapsed=0;



  rc = setitimer(ITIMER_REAL, &itv, 0);
  detour_playback_start_tick = rdtsc();
  assert( rc == 0 );
}

/**
 * interval_usec + detour_usec > 19 usec
 * 
 * On BG/L CNK, interval timer has a delay between time when the timer
 * has expired and time when the signal handler has been called.  It
 * is approx. 10325 ticks delay in 699MHz cpu (relatively
 * long). During delay, a computation process losts process time.
 * 
 * Updating new timer value also takes time. It's approx. 2535 ticks.
 * 
 * (10325+2535)/699 = 18.4 usec
 */



