/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: selfish_detour_signal_overhead.c,v 1.4 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7 
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
/* ===================================
   measure the overhead on timer signal
   =================================== */
#include <stdlib.h>
#include <sys/time.h>
#include <signal.h>
#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <errno.h>

#include <stdarg.h>
#include <mpi.h>

#include "selfish_utils.h"

#ifdef BLRTS_DECREMENTER_HIJACK
#include <cnkhack.h>
#endif

#define   X_FLOPS          (1e+07)     // 10 MFLOPS
#define   FIXED_INTERVAL   (1000)      // 1 ms

static unsigned handler_cnt=0;

static void handler(int sig)
{
  handler_cnt ++;
}

unsigned selfish_detour_get_signal_overhead_ticks()
{
  struct itimerval itv;
  unsigned long long start,stop;
  double x;
  unsigned long long elapsed0,elapsed1;
  unsigned handler_cnt1;
  double   signal_overhead;
  int rc;

  /* Set the timer to zero, turning everything off */
  itv.it_value.tv_sec   = 0;
  itv.it_value.tv_usec  = 0;
  itv.it_interval       = itv.it_value;
  setitimer(ITIMER_REAL, &itv, 0);

  /* -----------------------
     Without timer interrupt
     ----------------------- */

  /* warming up */
  handler_cnt = 0;  x=0.0;
  start = rdtsc();
  while( x < X_FLOPS ) {  x = x + 1.0;  }  /* warm up */
  stop = rdtsc();

  handler_cnt = 0;  x=0.0;
  start = rdtsc();
  while( x < X_FLOPS ) {  x = x + 1.0;  }
  stop = rdtsc();

  elapsed0 = stop-start;
  
  /* -----------------------
     With timer interrupt
     ----------------------- */
  itv.it_value.tv_sec   = 0;
  itv.it_value.tv_usec  = FIXED_INTERVAL;
  itv.it_interval       = itv.it_value;

#ifndef BLRTS_DECREMENTER_HIJACK
  signal( SIGALRM, handler);
#else
  hijack_decrementer_handler( (void (*)()) handler);
#endif

  rc = setitimer(ITIMER_REAL, &itv, 0);
  assert( rc == 0 );

  /* 
    then, do the fixed amount of work here
  */
  handler_cnt = 0;  x=0.0;
  start = rdtsc();
  while( x < X_FLOPS ) {  x = x + 1.0;  }  /* warm up */
  stop = rdtsc();

  handler_cnt = 0;  x=0.0;
  start = rdtsc();
  while( x < X_FLOPS ) {  x = x + 1.0;  }
  stop = rdtsc();

  elapsed1 = stop-start;
  handler_cnt1 = handler_cnt;

  signal_overhead = (double)(elapsed1-elapsed0)/handler_cnt1;

  return (unsigned)( signal_overhead );
}
