/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: selfish_utils.c,v 1.7 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
#include "selfish_utils.h"

/**
 *
 * cpufreq() returns cpu frequency in HZ.  It requires cpu tick
 * counter which is incremented every 1 Hz.
 * 
 * Pentimum class cpu and powerpc has tick counter.
 *
 */

uint64_t  cpufreq(void)
{
  uint64_t start_cycle, stop_cycle, tdiff;
  struct timeval start_tv, stop_tv;
  uint64_t i, trycnt, cyclediff;;
  uint64_t freq;

  static uint64_t _cpufreq = 0;

  if( _cpufreq > 0 ) return _cpufreq;

  for( trycnt=0; trycnt<3; trycnt++) {
    /* for psuedo random generator */
    double v=0;
    unsigned int r_a = 1588635695, r_m = 4294967291U, r_q = 2, r_r = 1117695901;
    unsigned int r_seed = 93186752;

    /* start timer */
    gettimeofday( &start_tv, 0 );
    start_cycle = rdtsc();

    /* this calculation doesn't mean anything */
    /* it just makes cpu busy by floating operations  */
    for(i=0;i<1000*1000*2;i++) {
      r_seed = r_a*(r_seed % r_q) - r_r*(r_seed / r_q);
      v = (v + ((double)r_seed / (double)r_m))/2.0;
    }

    stop_cycle = rdtsc();
    gettimeofday( &stop_tv, 0 );

    /* usec */
    tdiff =
      ( stop_tv.tv_sec  * 1000 * 1000 + stop_tv.tv_usec ) -
      ( start_tv.tv_sec * 1000 * 1000 + start_tv.tv_usec );
  
    cyclediff = stop_cycle - start_cycle;

    // freq = (cyclediff/tdiff) * 1000 +  (cyclediff%tdiff)*1000 / tdiff;  // KHz

/* round out freq. */
    freq = (cyclediff/tdiff) * 1000;
    freq *= 1000; // Hz

/* this one is more accurate
    freq = cyclediff * 1000 * 1000;
    freq /= tdiff;
*/

    if( _cpufreq < freq ) _cpufreq = freq;
  }

  return _cpufreq;
}



/**
 * a setitimer wrapper for convenience
 * it doesn't set tv_sec
 */
void _set_itimer_usec(long tv_usec, void (*handler)(int) )
{
  struct itimerval itv;
  int rc;

  itv.it_value.tv_sec  = 0;
  itv.it_value.tv_usec = tv_usec;
  itv.it_interval      = itv.it_value;

  signal(SIGALRM, handler);
  rc = setitimer(ITIMER_REAL, &itv, 0);
  assert( rc == 0 );

  if( tv_usec ) {
    printf("set itervel timer. %ld usec.\n", tv_usec); 
  } else {
    printf("reset itervel timer.\n");
  }
}


/*  $RCSfile: selfish_utils.c,v $   $Author: kazutomo $
 *  $Revision: 1.7 $  $Date: 2007/01/03 21:16:49 $
 */

