/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: selfish_utils.h,v 1.14 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
#ifndef __SELFISH_UTILS_H_DEFINED__
#define __SELFISH_UTILS_H_DEFINED__

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <signal.h>
#include <sys/time.h>
#include <math.h>
#include <ctype.h>

/* ----------------------------------- */

#if defined(__i386__) 

typedef unsigned long long int ticks_t;

static __inline__ ticks_t rdtsc(void)
{
  unsigned long long int x;
     __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
     return x;
}
#elif defined(__x86_64__)

typedef unsigned long long int ticks_t;

static __inline__ ticks_t rdtsc(void)
{
  unsigned hi, lo;
  __asm__ __volatile__ ("rdtsc" : "=a"(lo), "=d"(hi));
  return ( (ticks_t)lo)|( ((ticks_t)hi)<<32 );
}

#elif defined(__powerpc__)

typedef unsigned long long int ticks_t;

//static __inline__ ticks_t rdtsc(void)
static ticks_t rdtsc(void)
{
  unsigned long long int result=0;
  unsigned long int upper, lower,tmp;
  __asm__ __volatile__(
                "0:                \n\t"
                "\tmftbu   %0      \n\t"
                "\tmftb    %1      \n\t"
                "\tmftbu   %2      \n\t"
                "\tcmpw    %2,%0   \n\t"
                "\tbne     0b      \n\t"
		: "=r"(upper),"=r"(lower),"=r"(tmp)
		);
  result = upper;
  result = result<<32;
  result = result|lower;

  return(result);
}

#else

#error "No tick counter is available!"

#endif  // architecture


/* ----------------------------------- */


/*
  simple_pi_calc() does PI(3.1415..) calculation using a most
  primitive and simplest pi calculation. It may be useful to check
  floating point perfomance.

  x is a floating point variable. the result is stored to x.  NN is
  iteration of loop. Bigger NN make the result more close to PI.
 */
#define  selfish_simple_pi_calc(x,NN)	\
  do { \
    int __i;				\
    (x) = 0.0; \
    for(__i=0;__i<(NN);__i++) {				  \
      if( (__i%2)==0 ) (x)+=(1.0/(1.0+2.0*(double)__i)) ; \
      else  (x) -= (1.0/(1.0+2.0*(double)__i)); \
    } \
    (x) *= 4.0; \
  } while(0);




/**
 * cpufreq() detects cpu frequency using tick counter and returns it.
 * If the tick counter is incremented by external clock source,
 * cpufreq() returns external clock frequency , instead of internal
 * clock frequency.
 * 
 */
uint64_t  cpufreq(void);

void _set_itimer_usec(long tv_usec,void (*handler)(int) );

double  ticks_to_second(ticks_t t);

/* XXX: this is for BGL chip. so clear up it later  */




/*
  fast random number generators
*/
void seedMT();
unsigned long randomMT();


#endif

/*  $RCSfile: selfish_utils.h,v $   $Author: kazutomo $
 *  $Revision: 1.14 $  $Date: 2007/01/03 21:16:49 $
 */
