/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: test_selfish_detour.c,v 1.8 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/

#include <stdint.h>
#include "selfish_detour.h"
#include "mpi.h"

#define   X_FLOPS          (1e+08)

static uint64_t do_fixed_work()
{
  uint64_t start,stop;
  double x;
  x=0.0;
  start = rdtsc();
  while( x < X_FLOPS ) {  x = x + 1.0;  }
  stop = rdtsc();
  return stop - start;
}

extern const long get_detour_interval_usec(int *mode, int *detour_type);
extern const long get_detour_duration_usec(int *mode, int *detour_type);
extern const int  get_detour_count();
extern const int  get_n_detour_types();

int main(int argc, char* argv[])
{
  int rank,size;
  int mode = 0;
  int detour_type = 0;
  int randomized_start = 0;
  int detour_cnt;
  uint64_t  elapsed, elapsed_base;
  uint64_t  detour_interval, detour_duration;
  
  MPI_Init(&argc, &argv);
  MPI_Comm_rank(MPI_COMM_WORLD, &rank);
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Barrier(MPI_COMM_WORLD);

  printf("* do fixed work\n");
  elapsed_base = do_fixed_work();

  mode = 2;
  randomized_start = 1;

  for( detour_type = 0; detour_type < get_n_detour_types(&mode) ; detour_type++) {
    detour_interval = get_detour_interval_usec(&mode,&detour_type);
    detour_duration = get_detour_duration_usec(&mode,&detour_type);
    printf("* interval=%llu usec / duration = %llu usec\n",
	   detour_interval, detour_duration );

    start_detour_injection(&rank, &mode, &detour_type, &randomized_start ); 

    elapsed = do_fixed_work();
    detour_cnt = get_detour_count();
    selfish_detour_playback_finalize();
  
    printf("1. fixed work without detour : %llu usec\n", elapsed_base/700);
    printf("2. fixed work with    detour : %llu usec\n", elapsed/700);
    printf("detour cnt                   : %d\n", detour_cnt);
    printf("expected time for 2.         : %llu usec\n",
	   elapsed/700 - detour_cnt*detour_duration );
    
    printf("error                        : %f usec / signal \n",
	   ((double)elapsed_base/700.0 - (double)( (double)elapsed/700.0 - (double)detour_cnt*detour_duration ))/(double)detour_cnt );
    puts("");
  }

  return 0;
}
