//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: AnalyzeDetour.cpp,v 1.10 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************

/**
 *
 * AnalyzeDetour [options]
 *
 * -i filename : input data filename
 *               if -i is not specified, data is read from stdin
 *
 */

#include "AnalyzeDetour.h"
#include "StatsCalc.h"


AnalyzeDetourOptions::AnalyzeDetourOptions( int argc, char* argv[] )
{
  /* initialize variables */
  progname_m = argv[0];
  datafn_m.clear();
  verbose_m = 0;
  outputbn_m = "default_output_basename";
  enableSimulation_m = false;

  /* parse options */
  for(int i=1; i< argc; i++ ) {
    std::string s = argv[i];

    if( s == "-h" || s == "--help" ) {
      usage();
      exit(0);
    } else if( s == "-v" || s == "--verbose" ) {
      verbose_m ++;
    } else if( s == "-i" || s == "--input" ) {
      if( !( (i+1) < argc ) ) {
	std::cerr << "Too few parameters" << std::endl;
	usage();
	exit(0);
      }
      datafn_m = argv[++i];
    } else if( s == "-b" || s == "--basename" ) {
      if( !( (i+1) < argc ) ) {
	std::cerr << "Too few parameters" << std::endl;
	usage();
	exit(0);
      }
      outputbn_m = argv[++i];
    } else if( s == "-s" || s == "--simulation" ) {
      enableSimulation_m = true;
    }
  }
}


void AnalyzeDetourOptions::usage() const
{
  std::ostream& out = std::cout;

  out << std::endl;
  out << progname_m << " [options]" << std::endl;
  out << "[Options]" << std::endl;
  out << " -i : input data filename" << std::endl;
  out << "Note: if input data file is not specified, data is read from stdin" << std::endl;
}


int main(int argc, char *argv[])
{
  AnalyzeDetourOptions opts( argc, argv );
  
  return statscalc( opts );
}
