//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: DetourRawData.cpp,v 1.18 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************
#include "DetourRawData.h"

#include <iomanip>

/**
 *
 * parse() 
 *
 */

bool DetourData::parse(const std::string& fn_data) {
  if( !load( fn_data ) ) return false;
  
  std::map <std::string, std::string>::iterator it;
  
  for( it=param_m.begin(); it!=param_m.end(); ++it ) {
    // std::cout << (*it).first << std::endl;
    
    if( (*it).first == "CPU_FREQ_HZ" ) {
      hd_m.cpuFreq = atoll( (*it).second.c_str() );
    } else if( (*it).first == "START_TICK" ) {
      hd_m.startTick = atoll( (*it).second.c_str() );
    } else if( (*it).first == "LAST_TICK" ) {
      hd_m.lastTick = atoll( (*it).second.c_str() );
    } else if( (*it).first == "MIN_TICKS" ) {
      hd_m.minTicks = atoll( (*it).second.c_str() );
    } else if( (*it).first == "PRECISION" ) {
      hd_m.precision = atoll( (*it).second.c_str() );
    } else if( (*it).first == "MAX_LATENCY" ) {
      hd_m.maxLatency = atoll( (*it).second.c_str() );
    } else if( (*it).first == "MIN_LATENCY" ) {
      hd_m.minLatency = atoll( (*it).second.c_str() );
    } else if( (*it).first == "N_SEGS" ) {
      hd_m.nSegs = atoi( (*it).second.c_str() );
    } else if( (*it).first == "TOOL_VERSION" ) {
      hd_m.toolVersion = (*it).second ;
    }
  }
  
  convertDetourToRunSegs();
  
  return true;
}


bool DetourData::resample(double sec)
{
  tick_t q;

  if( sec > totalTimeLength() ) {
    std::cerr << "resample period is more than total recording length\n";
    return false;
  }

  q = (tick_t) ( sec * (double)hd_m.cpuFreq );

  // std::cout << "q=" << q << "(ticks)" << std::endl;


  std::map<tick_t,tick_t>  detourmap;

  for( int i=0;i<detourSize(); i++ ) {
    tick_t pos =  ( detourStartTick_m[i] / q ) * q;


    // /* calculate the average of detours in Q */
    // tick_t tmp = detourmap[ pos ];
    // detourmap[ pos ] = (tmp + detourDurationTick_m[i])/2;

    detourmap[ pos ] += detourDurationTick_m[i];




    /*
    std::cout << std::setw(12) << detourStartTick_m[i] << " , ";
    std::cout << std::setw(5)  << detourDurationTick_m[i] << "\t";
    std::cout << std::setw(12) << pos  << " , ";
    std::cout << std::setw(5)  << detourmap[ pos ] <<  std::endl;
    */
  }

  /**
   * re-assign to detour segs
   */

  detourStartTick_m.clear();
  detourDurationTick_m.clear();

  std::map<tick_t,tick_t>::const_iterator  it;

  // assume that the keys of map are sorted and 
  // forward iterator scans from a small value to big value key.

  for( it=detourmap.begin(); it!=detourmap.end(); ++it ) {
    detourStartTick_m.push_back( (*it).first );
    detourDurationTick_m.push_back( (*it).second );
  }
  
  convertDetourToRunSegs();

  resamplesize = sec;
  resampled = true;

  return true;
}



void DetourData::writeDetourTimeSeries( const char* filename ) 
{
  std::ostream*  output=0;

  if( filename == (const char*)0 ) {
    output = &std::cout;
  } else {
    output = new std::ofstream(filename , std::ios::out);
  }

  *output << "# N_SEGS=" << detourSize() << std::endl;
  *output << "# " << std::endl;
  *output << "# [comment]" << std::endl;
  *output << "# Each line contains one detour" << std::endl;
  *output << "# The first column is start time of detour" << std::endl;
  *output << "# The second one is duration time of detour" << std::endl;
  *output << "# Unit of time is second" << std::endl;
  *output << "# " << std::endl;



  if( resampled ) {
    *output << "# RESAMPLESIZE=" << resamplesize << std::endl;
  }

  for( int i=0;i<detourSize(); i++ ) {
    double ts,td;
    ts = (double)detourStartTick_m[i] / (double)hd_m.cpuFreq;
    td = (double)detourDurationTick_m[i] / (double)hd_m.cpuFreq;

    *output << ts << " " << td << std::endl;
  }

  if( output != &std::cout ) {
    delete output;
  }
}








void DetourData::writeDetourSort( const char* filename )
{
  std::ostream*  output=0;

  if( filename == (const char*)0 ) {
    output = &std::cout;
  } else {
    output = new std::ofstream(filename , std::ios::out);
  }

  Vec_t detour_tmp, detour_sort;

  assignDetourDurationInSec( detour_tmp );

  detour_sort.resize( detour_tmp.size() );
  std::copy( detour_tmp.begin(),  detour_tmp.end(), detour_sort.begin() );
  std::sort( detour_sort.begin(), detour_sort.end() );

  Vec_t::const_iterator it;

  for(it=detour_sort.begin(); it!=detour_sort.end(); ++it ) {
    *output << (*it) << std::endl;
  }

  if( output != &std::cout ) {
    delete output;
  }
}



//
// XXX: fix it. kind of quick and dirty implementation. 
void DetourData::writeDetourSortPercent( const char* filename, bool cpuscalefit )
{
  std::ostream*  output=0;

  if( filename == (const char*)0 ) {
    output = &std::cout;
  } else {
    output = new std::ofstream(filename , std::ios::out);
  }

  Vec_t detour_tmp, detour_sort;

  assignDetourDurationInSec( detour_tmp );

  detour_sort.resize( detour_tmp.size() );
  std::copy( detour_tmp.begin(),  detour_tmp.end(), detour_sort.begin() );
  std::sort( detour_sort.begin(), detour_sort.end() );

  StatsBasic  tmp_s;
  tmp_s.calc( detour_sort );
  double detour_total = tmp_s.total();
  double r = detour_total/totalTimeLength();

  Vec_t::const_iterator it;



  int idx=0;
  for(it=detour_sort.begin(); it!=detour_sort.end(); ++it ) {

    double pos = ( r*(double)idx/(double)detour_sort.size()) + (1.0-r);
    pos = pos * 100.0;
    idx++;

    if( cpuscalefit == false ) {
      *output << 
	std::setprecision(30) << pos << " " << 
	std::setprecision(30) << (*it) << std::endl;    
    } else {
      //double ttt = ((*it) * hd_m.cpuFreq) *  
      //( (double)hd_m.cpuFreq / 1000000000.0 );
      //double ttt = (*it) * hd_m.cpuFreq;

      double ttt = (*it) * (double)hd_m.cpuFreq / (double)hd_m.minTicks;

      *output << 
	std::setprecision(30) << pos << " " <<  
	std::setprecision(30) << ttt << std::endl;    
    }
    
  }



  if( output != &std::cout ) {
    delete output;
  }
}





void DetourData::convertDetourToRunSegs()
{
  // conver detour segs to run segs
  
  runStartTick_m.clear();
  runDurationTick_m.clear();
  
  if( detourStartTick_m.size() > 0 ) {
    runStartTick_m.push_back(0);
    runDurationTick_m.push_back(detourStartTick_m[0]);
  
    for( int i=1;i<detourSize(); i++ ) {
      tick_t s = detourStartTick_m[i-1] + detourDurationTick_m[i-1];

      runStartTick_m.push_back( s );
      runDurationTick_m.push_back(detourStartTick_m[i]-s);
    }
  } else {
    std::cerr << "detourStartTick_m is zero" << std::endl;
  }
}




/* ==================================================================== */


/**
 *
 * load() : read data and store 
 *
 */

bool DetourRawData::load( const std::string& fn_data )
{
  std::istream*  data_in=0;
  
  /**
   * open trick
   */
  if( fn_data.empty() ) {
    data_in = &std::cin;
    std::cerr << "NOTE: reading data from stdin" << std::endl;
  } else {
    data_in = new std::ifstream( fn_data.c_str() );
  }

  assert( data_in );
  if( !data_in->fail() ) {
    std::cout << "Failed to open " << fn_data.c_str() << "\n";
  }
  assert( !data_in->fail() );

  /**
   * parse data file
   */
  char buf[80];

  class PosixRegexWrap hd_reg( "^#[ ]+([^=]+)=([0-9.]+)" );
  class PosixRegexWrap data_reg( "^([0-9]+)[ ]+([0-9]+)" );

  while( !data_in->eof() ) {
    (*data_in).getline(buf, sizeof(buf));

    if( data_reg.match( buf ) ) {
      std::string t1,t2;
      
      assert( data_reg.get(1,t1) && data_reg.get(2,t2) );
    
      detourStartTick_m.push_back(    atoll( t1.c_str() ) );
      detourDurationTick_m.push_back( atoll( t2.c_str() ) );
    } else {
      if( hd_reg.match( buf )  ) {
	std::string key, val;

	if( hd_reg.get(1,key) && hd_reg.get(2,val) )  {
	  param_m[key] = val;
	}
      }
    }
  }


  // delete file object only when data is read from file
  if( data_in != &std::cin ) {
    delete data_in;
  }

  return true;
}





