//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: DetourRawData.h,v 1.16 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************
#ifndef __DETOURRAWDATA_DEFINED__
#define __DETOURRAWDATA_DEFINED__

#include <iostream>
#include <fstream>
#include <string>

#include <assert.h>

#include <map>
#include <vector>

#include <stdint.h>

#include "PosixRegexWrap.h"

#include "StatsBasic.h"

/**
 * DetourRawData is a container for Detour data.  It provides a low
 * level file load method and other basic methods.
 */ 

class DetourRawData {
public:
  typedef  uint64_t  tick_t;

  DetourRawData() { }
  ~DetourRawData() { }

  bool load( const std::string& fn_data );

  std::string& header(std::string  key) {
    return param_m[key];
  }

  tick_t header_tick(std::string  key) {
    return atoll(param_m[key].c_str());
  }

  /**
   * check see if key is defined in map
   */
  bool key_defined(std::string& key) {
    std::map <std::string, std::string>::iterator iter;

    iter = param_m.find( key );
    if( iter == param_m.end() ) return false;

    return true;
  }

  int  detourSize( ) {
    return detourDurationTick_m.size();
  }

  tick_t detourStartTick( int i ) {
    return detourStartTick_m[i];
  }

  tick_t detourDurationTick( int i ) {
    return detourDurationTick_m[i];
  }

 protected:
  /** header section */
  std::map <std::string, std::string>  param_m;

  /** data section */
  std::vector<tick_t>  detourStartTick_m;
  std::vector<tick_t>  detourDurationTick_m;
 private:
};


/**
 * DetourData class is based on DetourRawData class.  It provides data
 * analysis some methods. The methods in this class tends to use
 * Second as unit. For efficiency, this class holds StatsBasic
 * objects.
 */ 

class DetourData : public DetourRawData {
 public:
  DetourData()  { resampled = false;   resamplesize=0.0;  }
  
  DetourData(const DetourData& obj) { 
    // duplicate all local storage
    
    resampled = false;

    hd_m = obj.hd_m;

    runStartTick_m.resize( obj.runStartTick_m.size() );
    std::copy( obj.runStartTick_m.begin(),
	       obj.runStartTick_m.end(),
	       runStartTick_m.begin() );

    runDurationTick_m.resize( obj.runDurationTick_m.size() );
    std::copy( obj.runDurationTick_m.begin(),
	       obj.runDurationTick_m.end(),
	       runDurationTick_m.begin() );

    // copy objs defined in the super class
    param_m = obj.param_m;

    detourStartTick_m.resize( obj.detourStartTick_m.size() );
    std::copy( obj.detourStartTick_m.begin(),
	       obj.detourStartTick_m.end(),
	       detourStartTick_m.begin() );

    detourDurationTick_m.resize( obj.detourDurationTick_m.size() );
    std::copy( obj.detourDurationTick_m.begin(),
	       obj.detourDurationTick_m.end(),
	       detourDurationTick_m.begin() );

    // StatsBasic
  }
  
  ~DetourData() {}

  typedef std::vector<double> Vec_t;

  bool parse(const std::string& fn_data);

  //bool DetourData::resample(double sec);
  bool resample(double sec);

  void assignDetourDurationInSec( Vec_t& v ) {
    v.clear();

    std::vector<tick_t>::const_iterator it;
    for( it=detourDurationTick_m.begin();
	 it!=detourDurationTick_m.end();
	 ++it ) {
      v.push_back( (double)(*it)/(double)hd_m.cpuFreq );
    }
  }

  void assignRunDurationInSec( Vec_t& v ) {
    v.clear();

    std::vector<tick_t>::const_iterator it = runDurationTick_m.begin();
    if( (*it) == 0 ) it++;

    for( ; it!=runDurationTick_m.end(); ++it ) {
      v.push_back( (double)(*it)/hd_m.cpuFreq );
    }
  }

  // write series method convert data unit from tick to second
  void writeDetourTimeSeries( const char* filename );
  void writeDetourSort( const char* filename ); 
  void writeDetourSortPercent( const char* filename, bool cpuscalefit = false );

  // void writeDetourHisto( const char* fn, int n_bucket=10);



  /**
   * reporting method
   */ 
  void report( std::ostream& out = std::cout ) {
    out << "DATA_N=" << hd_m.nSegs << std::endl;
    out << "DATA_MAX_DETOUR=" << ((double)hd_m.maxLatency/hd_m.cpuFreq) << std::endl;
    out << "DATA_MIN_DETOUR=" << ((double)hd_m.minLatency/hd_m.cpuFreq) << std::endl;
    out << "DATA_MIN_LOOP_TIME=" << ((double)hd_m.minTicks  /hd_m.cpuFreq) << std::endl;
    out << "DATA_DETOUR_THRESHOLD=" << ((double)hd_m.precision /hd_m.cpuFreq)  << std::endl;
  }

  double totalTimeLength() {
    
    return 
      (double)( hd_m.lastTick + detourDurationTick_m.back() - hd_m.startTick )
      /
      (double)hd_m.cpuFreq;
  }

 protected:

 private:
  struct _hd {
    tick_t cpuFreq, startTick, lastTick;
    tick_t minTicks, precision, maxLatency, minLatency;
    int    nSegs;
    std::string  toolVersion;
  } hd_m;

  /** 
   * runStartTick_m and runDurationtick_m are converted from
   * detourStartTick_m and detourDurationTick_m 
   */
  std::vector<tick_t> runStartTick_m;
  std::vector<tick_t> runDurationTick_m; 

  void convertDetourToRunSegs();

  bool   resampled;
  double resamplesize;

  Vec_t detourDurationStats_m;
  Vec_t runDurationStats_m;
};



#endif
