//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: PosixRegexWrap.h,v 1.3 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************
#ifndef __POSIXREGEXWRAP_DEFINED__
#define __POSIXREGEXWRAP_DEFINED__

#include <iostream>

#include <sys/types.h>
#include <regex.h>

#define DEBUG (1)

/**
 * 
 * a wrapper class for POSIX regex
 *
 */

class PosixRegexWrap {
public:
  PosixRegexWrap(const char* regex, int flags=REG_EXTENDED, int _maxmatches=32 ) {
    maxmatches = 0;
    pmatch = 0;
    if( compile( regex, flags, _maxmatches ) == false ) {
#ifdef DEBUG
      std::cerr << "bad regex : " << regex << std::endl;
#endif
    }
  }

  ~PosixRegexWrap() { reset(); }

  void reset() {
    if(pmatch) {
      maxmatches = 0;
      delete[] pmatch;
    }
    if( ready ) { 
      regfree( &preg );
      ready = false;
    }
  }

  /**
   * compile()
   * also support recompiling regex
   */
  bool compile( const char* regex, int flags = REG_EXTENDED, 
		int _maxmatches=32 ) {

    maxmatches = _maxmatches;
    pmatch = new regmatch_t[maxmatches];

    if( ! pmatch ) {
      reset();
      return false;
    }

    if( regcomp( &preg, regex, flags )!=0 ) {
      reset();
      return false;
    }
    return true;
  }


  /**
   * match()
   *
   * return true if matched, otherwise false.
   */
  bool match( const char* str, int eflags=0 ) {
    nmatched = -1;

    s = str;

    if( regexec(&preg,str,
		maxmatches, pmatch, eflags) != 0  ) {
      return false;
    }

    nmatched = 0;
    while( nmatched<maxmatches && pmatch[nmatched].rm_so > -1 ) {
      nmatched ++;
    }

    return  true;
  }


  
  /**
   * get()
   *
   */

  bool get(int pos, std::string& token) {
    if( pos < 0 || pos >= nmatched ) return false;
    
    token = s.substr( pmatch[pos].rm_so, pmatch[pos].rm_eo-pmatch[pos].rm_so );

    return true;
  }


private:
  bool       ready;         /* true if ready to match */
  regex_t    preg;

  int        maxmatches;
  regmatch_t *pmatch;

  int        nmatched;

  std::string s;
};

#endif
