//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: StatsBasic.cpp,v 1.9 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************
#include <iostream>
#include <iterator>
#include <algorithm>
#include <limits>   // numeric_limits


#include "StatsBasic.h"

static void copy_sort( const StatsBasic::Vec_t& src, 
		       StatsBasic::Vec_t& dst )
{
  dst.resize( src.size() );
  std::copy( src.begin(), src.end(), dst.begin() );
  std::sort( dst.begin(), dst.end() );
}

void StatsBasic::calc( const Vec_t& vec )
{
  Vec_t::const_iterator it;

  double tmp;

  sorted_m.clear();
  max_m = min_m =  average_m =  variance_m = median_m = total_m = 0.0;
  valid_m = false;

  /* vec size should be more than 1 due to variation calculation */
  if( !(vec.size()>1)  ) return;

  copy_sort( vec , sorted_m );

  if( !(sorted_m.size()>1) ) return;

  median_m =  sorted_m[ sorted_m.size()/2 ];


  /**
   * calc min, max and total within one loop
   */
  max_m = - std::numeric_limits<double>::max();
  min_m =   std::numeric_limits<double>::max();

  tmp   = 0.0;
  for( it=vec.begin(); it!=vec.end(); ++it )  {
    tmp += (*it);
    if( (*it) > max_m ) max_m = (*it);
    if( (*it) < min_m ) min_m = (*it);
  }
  total_m = tmp;
  average_m = total_m/(double)vec.size();

  /**
   *   variance = (1/ (N-1) ) SIGMA ( Xi - Xm )^2
   *   standard deviation = sqrt( variance )
   */

  tmp = 0.0;
  for( it=vec.begin(); it!=vec.end(); ++it ) {
    tmp += ((*it) - average_m) * ((*it) - average_m) ;
  }
  /* unbiased estimate of the variance */
  variance_m =  tmp/(double)( vec.size() - 1 );

  
  valid_m = true;
}


void StatsBasic::report( const std::string& prefix,
			 std::ostream& out )
{
  out << prefix << "SIZE=" << sorted_m.size() << std::endl;
  out << prefix << "MIN=" << min_m << std::endl;
  out << prefix << "MAX=" << max_m << std::endl;
  out << prefix << "MAX_MIN=" << (max_m-min_m) << std::endl;
  out << prefix << "AV=" << average_m << std::endl;
  out << prefix << "VAR=" << variance_m << std::endl;
  out << prefix << "SD=" << sqrt(variance_m) << std::endl;
  out << prefix << "MEDIAN=" << median_m << std::endl;
  out << prefix << "TOTAL=" << total_m << std::endl;
}
