//***************************************************************************
// SELFISHDETOUR:selfishdetour-info
//     This file is part of Selfish Detour: OS detour(noise) benchmark
// SELFISHDETOUR:selfishdetour-info
//
// SELFISHDETOUR:selfishdetour-fillin
//     $Id: StatsBasic.h,v 1.6 2007/01/03 21:16:49 kazutomo Exp $
//     SelfishDetour_Version: 1.0.7
//     SelfishDetour_Heredity: FOSS_ORIG
//     SelfishDetour_License: GPL
// SELFISHDETOUR:selfishdetour-fillin
//
// SELFISHDETOUR:selfishdetour-gpl
//      Copyright: Argonne National Laboratory, Department of Energy,
//                 and University of Chicago.  2004, 2005, 2006, 2007
//      SelfishDetour License: GPL
// 
//      This software is free.  See the file license-files/license.GPL
//      for complete details on your rights to copy, modify, and use this
//      software.
// SELFISHDETOUR:selfishdetour-gpl
//***************************************************************************
#ifndef __STATSBASIC_H_DEFINED__
#define __STATSBASIC_H_DEFINED__

/**
 * provide class StatsBasic which does basic statically calculation
 * for input data
 *
 */

#include <vector>
#include <iterator>
#include <iostream>
#include <fstream>

#include <math.h>



class StatsBasic 
{
public:
  typedef std::vector<double>  Vec_t;

  StatsBasic() { valid_m = false; }

  StatsBasic(const Vec_t& vec ) {
    valid_m = false; 
    calc(vec);
  }

  ~StatsBasic() {}

  void calc( const Vec_t& vec );

  /**
   * return true if calc() finishes successfully
   */
  bool  valid() { return valid_m; }


  double max() { return max_m; }
  double min() { return min_m; }

  double average()    { return average_m; }

  /**
   * variance is unbiased
   *    (1/ (N-1) ) SIGMA ( Xi - Xm )^2
   */
  double variance()   { return variance_m; }

  /**
   * sdeviation is standard deviation
   */
  double sdeviation() { return sqrt(variance_m) ; }

  double total() { return total_m; }

  /**
   * reporting method
   */ 
  void report( const std::string& label,
	       std::ostream& out = std::cout );

  void dumpSortedData( const char* filename ) {
    dumptofile( filename, sorted_m );
  }

  /**
   * calculate histrogram and return array by deep-copying.
   * 
   * length of bucket is calculated automatically from min,max
   */
  Vec_t histogram( int n_buckets=10 ) {
    histo_m.clear();
    histo_m.resize(n_buckets);

    double step = (max_m - min_m) / (double)n_buckets;
    int p;

    Vec_t::const_iterator it;
    for( it=sorted_m.begin(); it!=sorted_m.end(); ++it ) {
      p = (int) ( (*it)/step );
      histo_m[p] ++;
    }

    return histo_m;
  }

  void dumpHistogram( const char* filename ) {
    dumptofile( filename, histo_m );
  }


private:
  double max_m, min_m;
  double average_m, variance_m;
  double median_m;
  double total_m;

  Vec_t sorted_m;

  bool valid_m;

  Vec_t histo_m; 


  void dumptofile( const char* filename, Vec_t& data ) {
    std::ostream*  output=0;

    if( filename == (const char*)0 ) {
      output = &std::cout;
    } else {
      output = new std::ofstream(filename , std::ios::out);
    }

    /**/
    Vec_t::const_iterator it;

    for( it=data.begin(); it!=data.end(); ++it ) {
      (*output) << (*it) << std::endl;
    }

    if( output != &std::cout ) {
      delete output;
    }
  }
};

#endif

