#!/usr/bin/perl

#***************************************************************************
# SELFISHDETOUR:selfishdetour-info
#      This file is part of Selfish Detour: OS detour(noise) benchmark
# SELFISHDETOUR:selfishdetour-info
#
# SELFISHDETOUR:selfishdetour-fillin
#      $Id: detour-ratio.pl,v 1.6 2007/01/03 21:16:49 kazutomo Exp $
#      SelfishDetour_Version: 1.0.7
#      SelfishDetour_Heredity: FOSS_ORIG
#      SelfishDetour_License: GPL
# SELFISHDETOUR:selfishdetour-fillin
#
# SELFISHDETOUR:selfishdetour-gpl
#       Copyright: Argonne National Laboratory, Department of Energy,
#                  and University of Chicago.  2004, 2005, 2006, 2007
#       SelfishDetour License: GPL
#  
#       This software is free.  See the file license-files/license.GPL
#       for complete details on your rights to copy, modify, and use this
#       software.
# SELFISHDETOUR:selfishdetour-gpl
#***************************************************************************

use strict;
use English;

my $cpufreq=0;
my $start_tick=0;
my $last_tick=0;
my $total=0;
my $max = 0;
my $min = 2**64;

my @data = ();

while(<>) {
  if( /^# CPU_FREQ_HZ=([0-9]+)/ ) {
    $cpufreq=$1;
  } elsif( /^# START_TICK=([0-9]+)/ ) {
    $start_tick=$1;
  } elsif( /^# LAST_TICK=([0-9]+)/ ) {
    $last_tick=$1;
  } elsif( /([0-9]+)\s+([0-9]+)/ ) {
    push @data, { t => $1, d => $2 };

    my $v = $2;

    $total += $v;
    if( $max < $v ) { $max = $v; }
    if( $min > $v ) { $min = $v; }
  }
}

my @detour = ();

foreach my $d (@data) {
  push @detour, $d->{d};
}

my @detour_sorted = sort { $a <=> $b } @detour;
my $detour_average = 0;
foreach my $d (@detour_sorted) {
#   print "$d\n";
   $detour_average += $d;
}
my $n = $#detour_sorted + 1 ;
$detour_average = $detour_average / $n;

my $detour_median = $detour_sorted[$n/2];


my $va=0.0;
foreach my $d (@detour_sorted) {
   $va = ($d - $detour_average ) * ($d - $detour_average );
}
$va = $va / ($n-1.0);
$va = sqrt($va);


my $elapsed = $last_tick - $start_tick;


$total/=$cpufreq;
$elapsed/=$cpufreq;


print  "total detour = $total   (sec)\n";
print  "total time   = $elapsed (sec)\n";
#printf "detour per.  = %8.6f %\n", $total*100.0/$elapsed;

printf "%8.6f %% / %8.2f usec / %8.2f usec\n", 
$total*100.0/$elapsed, 
$min*1000*1000/$cpufreq, 
$max*1000*1000/$cpufreq;
printf "detour average = %8.2f usec / median = %8.2f usec / %8.2f \n", 
$detour_average*1000*1000/$cpufreq, 
$detour_median*1000*1000/$cpufreq,
$va*1000*1000/$cpufreq;





# memo
# for t in 000 001 002 003 ; do echo "[${t}]" ; for i in *_${t}_detour.dat
# ; do echo $i : `../../../detour-ratio.pl < $i` ; done ; done > seaborg-1.txt

