/****************************************************************************/
/* SELFISHDETOUR:selfishdetour-info */
/*     This file is part of Selfish Detour: OS detour(noise) benchmark
 */
/* SELFISHDETOUR:selfishdetour-info */
/* */
/* SELFISHDETOUR:selfishdetour-fillin */
/*     $Id: gen-test-dat.c,v 1.6 2007/01/03 21:16:49 kazutomo Exp $
 *     SelfishDetour_Version: 1.0.7
 *     SelfishDetour_Heredity: FOSS_ORIG
 *     SelfishDetour_License: GPL
 */
/* SELFISHDETOUR:selfishdetour-fillin */
/* */
/* SELFISHDETOUR:selfishdetour-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and University of Chicago.  2004, 2005, 2006, 2007
 *      SelfishDetour License: GPL
 * 
 *      This software is free.  See the file license-files/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* SELFISHDETOUR:selfishdetour-gpl */
/****************************************************************************/
/**
   generate simple test data

 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include <math.h>

static inline double _sin( double hz, double p, int i, int n )
{
  // p is in radian
  return sin( hz* (double)i/(double)n * 2.0 * M_PI + p );  // 1HZ in size n data
}



static void mixedsinwaves(double *data,int size)
{
  int i;
  for(i=0;i<size;i++)  {
    data[i] = 
      0.1 +                  // direct current
      0.1*_sin(1.0, 0, i, size)   +  // 1 HZ
      0.3*_sin(12,  0, i, size)  +  // 12 HZ
      0.6*_sin(440, 0, i, size)    // 440 HZ
      ;
  }
}


static void sin1HZ(double *data,int size)
{
  int i;
  for(i=0;i<size;i++)  {
    data[i] = 1.0*_sin(1.0, 0, i,size);
  }
}



static inline double _sinc( double hz, double p, int i, int n )
{
  double x;
  // p is in radian
  x = hz * (double)i/(double)n * 2.0 * M_PI + p;

  if( fabs(x) < M_PI/10 ) return 1.0;

  return sin(x)/x;  
}

static void sinctest1( double* data, int size )
{
  int i;

  for(i=0;i<size;i++)  {
    data[i] = 0.0;
    data[i] += 1.0*_sinc(10.0, -10.0*M_PI , i , size);
  }
}


static void cube( double* data, int size )
{
  int i,j;
  int hz = 10;

  for(i=0;i<hz;i++)  {
    for( j=0; j< size/hz; j++ ) {
      if( j<size/hz/2 ) {
	data[(size/hz)*i+j] = 0.0;	

      } else {
	data[(size/hz)*i+j] = 1.0;
      }
    }
  }
}


static void pulse( double* data, int size )
{
  int i;

  for(i=0;i<size;i++)  {
    data[i] = 0.0;
  }

  for(i=0;i<size;i+=size/10)  {
    data[i] = 1.0;
  }
}

static void exponential( double* data, int size )
{
  int i,j;

  for(i=0;i<size;i++)  {
    data[i] = 0;
    for( j=0; j<size; j+=size/20 ) {
      if( i >= j ) {
	data[i] += exp(  - (double)( i - j )/20.0 );
      }
    }
  }
}



struct generators {
  char  key;
  char  *description;
  void  (* func)(double *data, int size);
} gens[] = {
  { '1', "1HZ sin. 1.0 amplitude",  sin1HZ }, 
  { 'm', "mixed sin waves. 1HZ/0.1, 12HZ/0.3 and 0.6/440", mixedsinwaves },
  { 's', "sinc 10HZ, 1/n center", sinctest1 },
  { 'p', "10HZ pulse", pulse },
  { 'e', "exponential", exponential },
  { 'c', "cubic", cube },
  { '*', "default. 1HZ sin", sin1HZ },
};

#define  n_gens    (sizeof(gens)/sizeof(struct generators))

static void write_to(double *data,int size)
{
  int i;

  printf("# N=%d\n", size);
    
  for(i=0;i<size;i++)  {
    printf("%lf\n", data[i]);
  }
}



int main(int argc, char* argv[])
{
  int size = 1024;
  double *data;
  char mode;
  int i;

  if( argc < 2 ) {
    printf( "%s mode [size]\n", argv[0]);
    printf( "\n");
    printf( "mode\n");
    for(i=0; i<n_gens; i++) {
      printf("  %c : %s\n", 
	     gens[i].key, gens[i].description );
    }
    return 1;
  }

  mode = argv[1][0];
  if( argc >= 3 ) {
    size = atoi( argv[2]);
  }
  assert( size > 0 );

  data = (double*)malloc( sizeof(double)* size );

  for(i=0; i<n_gens-1; i++) {
    if( mode == gens[i].key ) {
      fprintf(stderr,"you've selected %s\n",
	      gens[i].description);
      
      (*gens[i].func)(data, size); 
      break;
    }
  }
  if( i==n_gens-1 ) {
    fprintf(stderr,"you've selected %s\n",
	    gens[i].description);
    (*gens[i].func)(data, size); 
  }

  write_to( data,size);

  fprintf(stderr,"done\n");

  return 0;
}
