#!/bin/sh

#***************************************************************************
# SELFISHDETOUR:selfishdetour-info
#      This file is part of Selfish Detour: OS detour(noise) benchmark
# SELFISHDETOUR:selfishdetour-info
#
# SELFISHDETOUR:selfishdetour-fillin
#      $Id: generate-graph-detour-run.sh,v 1.6 2007/01/03 21:16:49 kazutomo Exp $
#      SelfishDetour_Version: 1.0.7
#      SelfishDetour_Heredity: FOSS_ORIG
#      SelfishDetour_License: GPL
# SELFISHDETOUR:selfishdetour-fillin
#
# SELFISHDETOUR:selfishdetour-gpl
#       Copyright: Argonne National Laboratory, Department of Energy,
#                  and University of Chicago.  2004, 2005, 2006, 2007
#       SelfishDetour License: GPL
#  
#       This software is free.  See the file license-files/license.GPL
#       for complete details on your rights to copy, modify, and use this
#       software.
# SELFISHDETOUR:selfishdetour-gpl
#***************************************************************************

basedir=`dirname $0`
cwd=`cd ${basedir} ; pwd`

if [ ! -x `basename $0` ] ; then
    echo something wrong
fi

TMP_PREFIX="/var/tmp/tmp-generate-graph-detour-run-$$:"

if [ -z "$2" ] ; then 
    echo "Usage: `basename $0` DATAFILENAME OUTPUTBASE"
    echo ""
    echo "The following PDF files will be generated"
    echo ""
    echo "${OUTPUTBASE}-time.pdf   : time series"
    echo "${OUTPUTBASE}-detour.pdf : sorted by detour duration"
    echo "${OUTPUTBASE}-run.pdf    : sorted by runtime duration"
    echo ""
    exit 1
fi

DATAFILE=$1
KEYWORD=$2

# get value with inefficient way :->
CPU_FREQ=`cat $DATAFILE | grep "^# CPU_FREQ_HZ=" | sed -e "s/# CPU_FREQ_HZ=//"`
PRECISION=`cat $DATAFILE | grep "^# PRECISION=" | sed -e "s/# PRECISION=//"`
MIN_LOOP_TICKS=`cat $DATAFILE | grep "^# MIN_TICKS=" | sed -e "s/# MIN_TICKS=//"`
MAX_LATENCY=`cat $DATAFILE | grep "^# MAX_LATENCY=" | sed -e "s/# MAX_LATENCY=//"`
MIN_LATENCY=`cat $DATAFILE | grep "^# MIN_LATENCY=" | sed -e "s/# MIN_LATENCY=//"`
N_SEGS=`cat $DATAFILE | grep "^# N_SEGS=" | sed -e "s/# N_SEGS=//"`

LAST_TICK=`cat $DATAFILE | grep "^# LAST_TICK=" | sed -e "s/# LAST_TICK=//"`
START_TICK=`cat $DATAFILE | grep "^# START_TICK=" | sed -e "s/# START_TICK=//"`


echo [Information]
echo ""
echo "CPU_FREQ = $CPU_FREQ"
echo "MIN LOOP TICKS = $MIN_LOOP_TICKS"
echo "PRECISION = $PRECISION"
echo "MAX LATENCY = $MAX_LATENCY"
echo "MIN LATENCY = $MIN_LATENCY"
echo "N_SEGS = $N_SEGS"
echo ""


XRANGEMAX=`expr $N_SEGS + $N_SEGS / 100 + $N_SEGS / 100`


RECORDEDTIME=`echo "(${LAST_TICK}-${START_TICK})/$CPU_FREQ" | bc -l`
RECORDEDTIMESTR=`printf "%.3f sec\n" $RECORDEDTIME`





$basedir/sortlen.pl < $DATAFILE > ${TMP_PREFIX}detour-$KEYWORD.dat
$basedir/reconstruct-run-segs.pl < $DATAFILE | $basedir/sortlen.pl > ${TMP_PREFIX}run-$KEYWORD.dat

# --------------------
#
# generate detour pdf
#
# --------------------

cat - > ${TMP_PREFIX}gnuplotcmd <<EOF
set terminal postscript
set output "${TMP_PREFIX}detour-$KEYWORD.ps"

set title "sorted detour time : $KEYWORD\nCPU_FREQ=$CPU_FREQ HZ / PRECISION=$PRECISION (ticks)\nMIN_LOOP_TICKS=$MIN_LOOP_TICKS (ticks) / MIN_LATENCY=$MIN_LATENCY (ticks) / MAX_LATENCY=$MAX_LATENCY (ticks)\n RECORDING_TIME=$RECORDEDTIMESTR"

set xlabel "data rank"
set ylabel "detour(tick)"

set xrange [0:$XRANGEMAX]
#set yrange [0:2000]
pl '${TMP_PREFIX}detour-$KEYWORD.dat'  title '$DATATITLE' w i

EOF

gnuplot ${TMP_PREFIX}gnuplotcmd
(cd /var/tmp ; ps2pdf  ${TMP_PREFIX}detour-$KEYWORD.ps )
cp      ${TMP_PREFIX}detour-$KEYWORD.pdf     $KEYWORD-detour.pdf

echo "$KEYWORD-detour.pdf has been generated." 


# --------------------
#
# generate time series pdf
#
# --------------------

cat - > ${TMP_PREFIX}gnuplotcmd <<EOF
set terminal postscript
set output "${TMP_PREFIX}time-$KEYWORD.ps"

#set terminal epslatex
#set output "${TMP_PREFIX}detour-$KEYWORD.eps"

set title "time series : $KEYWORD\nCPU_FREQ=$CPU_FREQ HZ / PRECISION=$PRECISION (ticks)\nMIN_LOOP_TICKS=$MIN_LOOP_TICKS (ticks) / MIN_LATENCY=$MIN_LATENCY (ticks) / MAX_LATENCY=$MAX_LATENCY (ticks)\n RECORDING_TIME=$RECORDEDTIMESTR"

set xlabel "time(tick)"
set ylabel "detour(tick)"

pl '$DATAFILE'  title '$DATATITLE' w i
#pl '$DATAFILE'  title '$DATATITLE' 

EOF

gnuplot ${TMP_PREFIX}gnuplotcmd
( cd /var/tmp ; ps2pdf ${TMP_PREFIX}time-$KEYWORD.ps )

cp ${TMP_PREFIX}time-$KEYWORD.pdf $KEYWORD-time.pdf

echo "$KEYWORD-time.pdf has been generated." 


if [ ! -z "${TMP_PREFIX}" ] ; then 
    rm -f ${TMP_PREFIX}*
fi


exit 0


# -------------------------
#
# generate process time pdf
#
# -------------------------

cat - > ${TMP_PREFIX}gnuplotcmd <<EOF
set terminal postscript
set output "${TMP_PREFIX}run-$KEYWORD.ps"

set title "sorted process timeslice(not accurate) : $KEYWORD\nCPU_FREQ=$CPU_FREQ HZ / PRECISION=$PRECISION (ticks)\nMIN_LOOP_TICKS=$MIN_LOOP_TICKS (ticks) / MIN_LATENCY=$MIN_LATENCY (ticks) / MAX_LATENCY=$MAX_LATENCY (ticks)\n RECORDING_TIME=$RECORDEDTIMESTR"

set xlabel "data rank"
set ylabel "process timeslice(tick)"

set xrange [0:$XRANGEMAX]
pl '${TMP_PREFIX}run-$KEYWORD.dat'  title '$DATATITLE' w i
#pl '${TMP_PREFIX}run-$KEYWORD.dat'  title '$DATATITLE' 

EOF

gnuplot ${TMP_PREFIX}gnuplotcmd
( cd /var/tmp ; ps2pdf ${TMP_PREFIX}run-$KEYWORD.ps )

cp ${TMP_PREFIX}run-$KEYWORD.pdf $KEYWORD-run.pdf

echo "$KEYWORD-run.pdf has been generated." 


