#!/usr/bin/perl

#***************************************************************************
# SELFISHDETOUR:selfishdetour-info
#      This file is part of Selfish Detour: OS detour(noise) benchmark
# SELFISHDETOUR:selfishdetour-info
#
# SELFISHDETOUR:selfishdetour-fillin
#      $Id: reporter.pl,v 1.16 2007/01/03 21:16:49 kazutomo Exp $
#      SelfishDetour_Version: 1.0.7
#      SelfishDetour_Heredity: FOSS_ORIG
#      SelfishDetour_License: GPL
# SELFISHDETOUR:selfishdetour-fillin
#
# SELFISHDETOUR:selfishdetour-gpl
#       Copyright: Argonne National Laboratory, Department of Energy,
#                  and University of Chicago.  2004, 2005, 2006, 2007
#       SelfishDetour License: GPL
#  
#       This software is free.  See the file license-files/license.GPL
#       for complete details on your rights to copy, modify, and use this
#       software.
# SELFISHDETOUR:selfishdetour-gpl
#***************************************************************************

use strict;


if ( $#ARGV == -1 ) {
   print "Usage: $0 topdir\n";
   exit 1;
}


my $topdir = $ARGV[0];

if ( ! -d $topdir ) {
  print "$topdir is not a valid directory\n";
}

if ( ! -f "${topdir}/data-spec" ) {
  print "$topdir/data-spec is required\n";
}

#
# setting for a dataset
#

# these two are for input
my $datadir = "${topdir}/output/";
my $origdatadir = "${topdir}/";
my $dataspecfile = "$origdatadir/data-spec";

# this is for output
my $outputdir = "${topdir}";

my $html_fn = "$outputdir/tables.html";


#
# read data set specificationa and description

my $dataspec = eval( `cat $dataspecfile` );

#
# 
# 

my $psimflag = 0;
if( -f "$datadir/parallel-simulation" ) {
  $psimflag = 1;
  print "psim : on\n";
} else {
  print "psim : off\n";
}


#
# read stats information for each data
#
foreach my $k ( keys %{ $dataspec } ) {
   my $statsres = read_statsres( $k );
   $dataspec->{$k}->{stats} = $statsres;

   if( $psimflag ) {
     my $at64k = read_at64k( $k );
     $dataspec->{$k}->{at64k} = $at64k;
   }

# prode option later to switch
  generate_graphs( $statsres, $k, $dataspec->{$k}->{desc} );
}



#
#  write html file
#


open( HTML, ">$html_fn" ) or die "Unable to open $html_fn";



print HTML '<html>' . "\n";

print HTML << "HEADER"
<head>
<meta content="text/html; charset=ISO-8859-1"
http-equiv="content-type">
<title></title>
<meta content="Kazutomo Yoshii &lt;kazutomo\@mcs.anl.gov&gt;" name="author">
</head>
HEADER
;

print HTML '<body>' . "\n";

# print HTML '<h2>System Detour Study</h2>' . "\n";


my $datatitle = "${topdir}/data-title.html";
if( -f "${datatitle}" ) {
  open( FH_R, "<${datatitle}"  ) or die "Unable to read ${datatitle}";
  while( <FH_R> ) {
    print HTML $_;
  }
  close( FH_R );
}


# print HTML '<table border="0">' . "\n";
# print HTML '<tr>' . "\n";
# print HTML '<td><img alt="" src="title.png"></td>';
# print HTML '<td>Comparison between Jazz and Seaborg on the estimated paralell factor. <br> This graph shows that Jazz is better characterics than Seaborg on the big cluster setting.</td>';
# print HTML '</tr>' . "\n";
# print HTML '</table>' . "\n";





#print HTML '<br>' . "\n";
#print HTML '<br>' . "\n";


#print HTML '<h3>Download</h3>' . "\n";
#print HTML '<br>' . "\n";

#print HTML '<td><a href="http://www-unix.mcs.anl.gov/fastos/software/selfish-detour-0.4.tar.bz2">selfish-detour-0.4.tar.bz2</a></td>' . "\n";
#print HTML ' : Download Detour Recorder, Analyer, Reported and pre-record datas.' . "\n";

#print HTML '<br>' . "\n";


print HTML '<h3>Data & Results</h3>' . "\n";

##
#
# start table here
#
##

print HTML '<center><b>Data Description</b></center>' . "\n";
print HTML '<table style="width: 100%; text-align: left;" border="1" cellpadding="2" cellspacing="2">' . "\n";

print HTML '<tbody>' . "\n";

print HTML '<tr>' . "\n";
print HTML '<td><b>#</b></td>' . "\n";
print HTML '<td><b>ID</b></td>' . "\n";
print HTML '<td><b>DESC</b></td>' . "\n";
print HTML '<td><b>CPU</b></td>' . "\n";
print HTML '<td><b>KERNEL</b></td>' . "\n";
print HTML '<td><b>Comment</b></td>' . "\n";
print HTML '<td><b>Data</b></td>' . "\n";
print HTML '<td><b>Timeseries</b></td>' . "\n";
print HTML '<td><b>Sorted</b></td>' . "\n";
print HTML '</tr>' . "\n";


foreach my $k (
	       sort { $dataspec->{$a}->{idx} <=> $dataspec->{$b}->{idx} }
	       keys %{ $dataspec } ) {
 print HTML '<tr>' . "\n";
 my $p = $dataspec->{$k};
 print HTML '<td>' . $p->{idx} . '</td>' .  "\n";

 print HTML '<td>' . $k . '</td>' .  "\n";

 print HTML '<td>' . $p->{desc} . '</td>' .  "\n";
 print HTML '<td>' . $p->{cpu} . '</td>' .  "\n";
 print HTML '<td>' . $p->{kernel} . '</td>' .  "\n";
 print HTML '<td>' . $p->{etc} . '</td>' .  "\n";

 my $link = "detour-${k}.dat";
 print HTML '<td><a href="' . $link . '">text</a></td>' . "\n";

 $link = "output/res-${k}-timeseries.png";
 print HTML '<td><a href="' . $link . '">png</a></td>' . "\n";

 $link = "output/res-${k}-sorted.png";
 print HTML '<td><a href="' . $link . '">png</a></td>' . "\n";

 print HTML '</tr>' . "\n";
}
print HTML '</tbody>' . "\n";
print HTML '</table>' . "\n";

print HTML '<br>' . "\n";
print HTML '<br>' . "\n";
print HTML '<br>' . "\n";


##
#
# end table here
#
##


##
#
# start table here
#
##


print HTML '<center><b>Stats Results</b></center>' . "\n";
print HTML '<table style="width: 100%; text-align: left;" border="1" cellpadding="2" cellspacing="2">' . "\n";

print HTML '<tbody>' . "\n";

print HTML '<tr>' . "\n";
print HTML '<td><b>#</b></td>' . "\n";
print HTML '<td><b>ID</b></td>' . "\n";
print HTML '<td><b>SIZE</b></td>' . "\n";
print HTML '<td><b>Ratio(%)</b></td>' . "\n";
print HTML '<td><b>Max(sec)</b></td>' . "\n";
print HTML '<td><b>Average(sec)</b></td>' . "\n";
print HTML '<td><b>Median(sec)</b></td>' . "\n";
print HTML '<td><b>Recording Time(sec)</b></td>' . "\n";
print HTML '<td><b>Other params</b></td>' . "\n";
print HTML '</tr>' .  "\n";

foreach my $k (
	       sort { $dataspec->{$a}->{idx} <=> $dataspec->{$b}->{idx} }
	       keys %{ $dataspec } ) {

 my $p = $dataspec->{$k};
 my $s = $dataspec->{$k}->{stats};

 print HTML '<tr>' . "\n";
 print HTML '<td>' . $p->{idx} . '</td>' .  "\n";
 print HTML '<td>' . $k . '</td>' .  "\n";


 print HTML '<td>' . $s->{DATA_N} . '</td>' .  "\n";

 my $percent = $s->{DETOUR_RUN_RATIO} * 100.0;
 print HTML '<td>' . $percent . '</td>' .  "\n";
 print HTML '<td>' . $s->{DATA_MAX_DETOUR} . '</td>' .  "\n";
 print HTML '<td>' . $s->{DETOUR_AV} . '</td>' .  "\n";
 print HTML '<td>' . $s->{DETOUR_MEDIAN} . '</td>' .  "\n";
 print HTML '<td>' . $s->{TOTAL_RECORDING_TIME} . '</td>' .  "\n";

 my $link = "output/res-${k}-stats.dat";
 print HTML '<td><a href="' . $link . '">text file</a></td>' . "\n";

 print HTML '</tr>' . "\n";
}

print HTML '</tbody>' . "\n";
print HTML '</table>' . "\n";

print HTML '<br>' . "\n";
print HTML '<br>' . "\n";
print HTML '<br>' . "\n";

##
#
# end table here
#
##


##
# start: timeseries sort image 
##

print HTML '<center><b>Graphs</b></center>' . "\n";
print HTML '<table style="width: 100%; text-align: left;" border="1" cellpadding="2" cellspacing="2">' . "\n";

print HTML '<tbody>' . "\n";

print HTML '<tr>' . "\n";
print HTML '<td><b>#</b></td>' . "\n";
print HTML '<td><b>Desc</b></td>' . "\n";
print HTML '<td><b>Timeseries</b></td>' . "\n";
print HTML '<td><b>Sorted</b></td>' . "\n";
print HTML '</tr>' . "\n";


foreach my $k (
	       sort { $dataspec->{$a}->{idx} <=> $dataspec->{$b}->{idx} }
	       keys %{ $dataspec } ) {
 print HTML '<tr>' . "\n";
 my $p = $dataspec->{$k};
 print HTML '<td>' . $p->{idx} . '</td>' .  "\n";

 print HTML '<td>' . $p->{desc} . '</td>' .  "\n";

 my $link = "output/res-${k}-timeseries.png";
 print HTML '<td><img alt="" src="' . $link . '"></td>' . "\n";

 my $link = "output/res-${k}-sorted.png";
 print HTML '<td><img alt="" src="' . $link . '"></td>' . "\n";

 print HTML '</tr>' . "\n";
}
print HTML '</tbody>' . "\n";
print HTML '</table>' . "\n";

print HTML '<br>' . "\n";
print HTML '<br>' . "\n";
print HTML '<br>' . "\n";


##
# start: timeseries sort image 
##



##
#
# start table here
#
##


if( $psimflag ) {

print HTML '<center><b>Results from the calculation of latency on the cluster system, using detour results from a single machine</b></center>' . "\n";
print HTML '<table style="width: 100%; text-align: left;" border="1" cellpadding="2" cellspacing="2">' . "\n";

print HTML '<tbody>' . "\n";

print HTML '<tr>' . "\n";
print HTML '<td><b>#</b></td>' . "\n";
print HTML '<td><b>ID</b></td>' . "\n";
print HTML '<td><b>3Hz@64K</b></td>' . "\n";
print HTML '<td><b>31Hz@64K</b></td>' . "\n";
print HTML '<td><b>316Hz@64k</b></td>' . "\n";
print HTML '<td><b>Data for 2D plotting</b></td>' . "\n";
print HTML '<td><b>Data for 3D plotting</b></td>' . "\n";
print HTML '<td><b>2D view</b></td>' . "\n";
print HTML '<td><b>3D view</b></td>' . "\n";
print HTML '</tr>' .  "\n";

foreach my $k (
	       sort { $dataspec->{$a}->{idx} <=> $dataspec->{$b}->{idx} }
	       keys %{ $dataspec } ) {

 my $p = $dataspec->{$k};
 my $s = $dataspec->{$k}->{stats};

 print HTML '<tr>' . "\n";
 print HTML '<td>' . $p->{idx} . '</td>' .  "\n";
 print HTML '<td>' . $k . '</td>' .  "\n";

 my $val;
 $val = $dataspec->{$k}->{at64k}->{"3.16228"};
 print HTML '<td>' . $val . '</td>' .  "\n";
 $val = $dataspec->{$k}->{at64k}->{"31.6228"};
 print HTML '<td>' . $val . '</td>' .  "\n";
 $val = $dataspec->{$k}->{at64k}->{"316.228"};
 print HTML '<td>' . $val . '</td>' .  "\n";
 

 my $link;

 $link = "output/res-${k}-2d.dat";
 print HTML '<td><a href="' . $link . '">text file</a></td>' . "\n";
 $link = "output/res-${k}-3d.dat";
 print HTML '<td><a href="' . $link . '">text file</a></td>' . "\n";


 $link = "output/res-${k}-2d.png";
 print HTML '<td><a href="' . $link . '">png file</a></td>' . "\n";
 $link = "output/res-${k}-3d-0.png";
 print HTML '<td><a href="' . $link . '">png file</a></td>' . "\n";

 print HTML '</tr>' . "\n";
}

print HTML '</tbody>' . "\n";
print HTML '</table>' . "\n";

print HTML '<br>' . "\n";
print HTML '<br>' . "\n";
print HTML '<br>' . "\n";

}  # if( $psimflag ) {


##
#
# end table here
#
##

print HTML 'For more information, e-mail to <a href="mailto:kazutomo@mcs.anl.gov">Kazutomo Yoshii <kazutomo@mcs.anl.gov></a>' . "\n";

print HTML '</body>' . "\n";
print HTML '</html>' . "\n";

close(HTML);

print "$html_fn has been generated.\n";
exit(0);

######################################################################
#
# subroutines
#
#######################################################################

sub read_statsres {
   my $id = shift;
   my %hash;

   my $fn = "${datadir}/res-${id}-stats.dat";

   open( FH, "<$fn") or die "Unable to open $fn";

   while(<FH>) {
     chomp;
     if( /([^= ]+)=([^= ]+)/ ) {
       # print $1 . " " . $2 . "\n";
       $hash{$1} = $2;
     }
   }
   close(FH);
   \%hash;
}


sub read_at64k {
  my $id = shift;
  my %hash;

  my $fn = "${datadir}/res-${id}-3d.dat";

  open( FH, "<$fn") or die "Unable to open $fn";

  while(<FH>) {
    chomp;
    if( /(\S+)\s+(\S+)\s+(\S+)\s+/ ) {
      if( $2 == 65536 ) {
	$hash{ $1 } = $3;
	# print "$1 $2 $3\n";
      }
    }
  }

  close(FH);
  \%hash;
}

sub generate_graphs {
   my $stats = shift;
   my $id = shift;
   my $desc = shift;



   #
   # generating timeseries graph
   #
   my $tsdata = "${datadir}/res-${id}-timeseries.dat";
   my $tsdata_png = "${datadir}/res-${id}-timeseries.png";
   my $margin = $stats->{"RUN_TOTAL"}/50.0;
   my $range_x_min = -$margin ; 
   my $range_x_max = $stats->{"RUN_TOTAL"} + $margin; 


   open(FH, ">tmp.gp") or die "Unable to open tmp.gp";
   print FH "set terminal png\n";
   print FH "set output \"$tsdata_png\"\n";
   print FH "set title \"$desc: timeseries\"\n";
   print FH "set xlabel \"Time(sec)\"\n";
   print FH "set ylabel \"Detour(sec)\"\n";
   print FH "set xrange [$range_x_min:$range_x_max]\n";
   print FH "plot \"${tsdata}\" title \"$id\" w i\n";
   print FH "repl\n";
   close(FH);
   `gnuplot tmp.gp`;

   #
   # generating sorted data graph
   #
   my $sorteddata = "${datadir}/res-${id}-sorted.dat";
   my $sorteddata_png = "${datadir}/res-${id}-sorted.png";

   my $n = $stats->{"DATA_N"};
   my $rangemin = - ($n/50.0);
   my $rangemax = $n + ($n/50.0);
   if( $n >= 1000 ) { 
     $rangemax = $n + 50; 
   }

   open(FH, ">tmp.gp") or die "Unable to open tmp.gp";
   print FH "set terminal png\n";
   print FH "set output \"$sorteddata_png\"\n";
   print FH "set title \"$desc: sorted by detour\"\n";
   print FH "set xlabel \"Rank\"\n";
   print FH "set ylabel \"Detour(sec)\"\n";
   print FH "set xrange [$rangemin:$rangemax]\n";
   print FH "plot \"${sorteddata}\" title \"$id\" w i\n";
   print FH "repl\n";
   close(FH);
   `gnuplot tmp.gp`;

}

