#!/usr/bin/perl

#***************************************************************************
# SELFISHDETOUR:selfishdetour-info
#      This file is part of Selfish Detour: OS detour(noise) benchmark
# SELFISHDETOUR:selfishdetour-info
#
# SELFISHDETOUR:selfishdetour-fillin
#      $Id: summarize-detours.pl,v 1.7 2007/01/03 21:16:49 kazutomo Exp $
#      SelfishDetour_Version: 1.0.7
#      SelfishDetour_Heredity: FOSS_ORIG
#      SelfishDetour_License: GPL
# SELFISHDETOUR:selfishdetour-fillin
#
# SELFISHDETOUR:selfishdetour-gpl
#       Copyright: Argonne National Laboratory, Department of Energy,
#                  and University of Chicago.  2004, 2005, 2006, 2007
#       SelfishDetour License: GPL
#  
#       This software is free.  See the file license-files/license.GPL
#       for complete details on your rights to copy, modify, and use this
#       software.
# SELFISHDETOUR:selfishdetour-gpl
#***************************************************************************

# summarize-detours.pl directory [targetname1 targetname2 ...]
# 
# directory is mandatory option. If targetname is omitted, target will
# be all files.
#
# This command depends on specific file name convention. The file name
# of original detour data should start with "detour-" and file
# extension should be ".dat".  for example, "detour-machine.dat" is a
# valid detour filename.  in this example, "machine" will be the
# detour basename which is used in data-spec file as ID.

use strict;

my $skip_sim = 1;

if ( $#ARGV == -1 ) {
   print "Usage: $0 topdir \n";
   exit 1;
}

my $_topdir = shift @ARGV;
my $topdir = `( cd $_topdir ; pwd)`;
chomp($topdir);

if ( ! -d $topdir ) {
  print "$topdir is not a valid directory\n";
}

if ( ! -f "${topdir}/data-spec" ) {
  print "$topdir/data-spec is required\n";
}


#
my $datadir = "${topdir}/output/";
my $origdatadir = "${topdir}";
my $dataspecfile = "$origdatadir/data-spec";

my $dataspec = eval( `cat $dataspecfile` );

my @target = ();

while( $#ARGV > -1 ) {
  my $tmp = shift @ARGV;
  push @target, $tmp;
}

if( $#target < 0 ) {
  print "use all target in definition file\n";
  foreach my $id (sort keys %{ $dataspec} ) {
    push @target, $id;
  }
}

# 
# checking see if target exists in data spec (for manual specification)
# 
foreach my $t (@target) {
  if( ! defined $dataspec->{"$t"} ) {
    print "$t is not defined in $dataspecfile\n";
    exit 1;
  }
}

die "no target!" if $#target == -1 ;

#
#
# making basic stats, simulation, graph
#



foreach my $t (@target) {
  print "Analyzing - $t\n";
  my $cmd = "./analyzer.sh $topdir b $t";
  my $cmdout = `$cmd`;
  if( ($?>>8) != 0 ) {
     print "./analyzer.sh aborted\n";
     print "[output]\n";
     print "$cmdout\n";
     exit 1;
  }
  print "$cmd" . "\n";

  if( ! $skip_sim ) {
    my $cmd = "./analyzer.sh $topdir s $t";
    `$cmd`;
    print "$cmd" . "\n";
    `touch ${topdir}/output/parallel-simulation`;
  }


  my $cmd = "./analyzer.sh $topdir g $t";
  `$cmd`;
  print "$cmd" . "\n";
}

print "Generating a reporting page...\n";
print `./reporter.pl $topdir`;


